from copy import deepcopy

conv2list = lambda x: x if type(x)==list else ([x,] if x else []) #converts all types to list type  

class ActorConfigs():
    '''
    All information to start all actors with different 
    pipeline_out and pipeline_inputs
    and for Kubernetes
    '''
    def __init__(self, 
                 nodes, 
                 monitor_reducer=None,
                 error_topic='fabrique.api.in',
                 pipeline_out='fabrique.api.in',
                 #pipeline_inputs=['fabrique.api.{pipeline_name}.out', 
                 #                 'fabrique.api.{pipeline_name}.{pipeline_version}.out'],
                 pipeline_inputs=['fabrique.api.{pipeline_name}.{pipeline_version}.out',],
                 pipeline_version='{pipeline_version}',
                 pipeline_name='{pipeline_name}',
                 topic_tpl='fabrique.{pipeline_name}.{pipeline_version}_{from_node}.out', 
                 topic_tpl_in='fabrique.{pipeline_name}.{pipeline_version}_{to_node}.in',
                 monitor_in=['fabrique.monitor.in',],
                 servers=None, 
                 local_repo_dir=None,
                 ):
        
        assert(nodes)
        self.nodes = nodes
        self.local_repo_dir = local_repo_dir
        self.error_topic = error_topic
        self.servers = servers
        self.pipeline_out = pipeline_out
        self.pipeline_inputs = [pi.format(pipeline_name=pipeline_name,
                                          pipeline_version=pipeline_version) for pi in pipeline_inputs]
        self.pipeline_name = pipeline_name
        self.pipeline_version = pipeline_version
        
        self.topic_tpl = topic_tpl
        self.topic_tpl_in = topic_tpl_in
        
        self.monitor_in = monitor_in
        self.monitor_reducer = monitor_reducer
        
    def set_pipeline_out(self, pipeline_out):
        self.pipeline_out = pipeline_out
    
    def set_pipeline_inputs(self, pipeline_inputs):
        self.pipeline_inputs = pipeline_inputs
    
    def get_kuber_annotation(self):
        pass
    
    def nodename2config(self, nodename):
        message_buses = self.make_message_bus_configs()        
        message_bus = message_buses[nodename]
        
        config = dict(message_bus = message_bus)
        config['message_bus']['servers'] = self.servers
        return config
    
    def make_message_bus_configs(self):
        message_buses = {}
        for nodename, node in self.nodes.items():
            message_buses[nodename] = {}
            
            if node['actor_type'] == 'detector':
                message_buses[nodename]['monitor_topics'] = ['fabrique.monitor.in',]
                topic_name = self.topic_tpl_in.format(to_node=nodename, 
                                                      pipeline_name=self.pipeline_name, 
                                                      pipeline_version=self.pipeline_version )
                
                message_buses[nodename]['in_topic'] = topic_name  
                if topic_name not in self.monitor_in:
                    self.monitor_in.append(topic_name)            
            
            for field in node:
                if field in ['engine_instance']:
                    continue
                message_buses[nodename][field] = deepcopy(node[field])
            
        for node_name, node in message_buses.items():
            actor_type = node['actor_type']

            #group_ids
            if 'reducer' not in actor_type:
                node['group_id'] = f"{self.pipeline_name}-{self.pipeline_version}"
            else:
                node['group_id'] = None
                
            node['client_id'] = node_name
            node['type'] = actor_type
            
            #out_topics -> in_topics
            if actor_type == 'dispatcher':
                node['out_topic_dict'] = {}
                for output in node['outputs']:
                    if output == 'end':
                        node['out_topic_dict'][output] = self.pipeline_out
                        continue
                    topic_name = self.topic_tpl_in.format(to_node=output, 
                                                          pipeline_name=self.pipeline_name, 
                                                          pipeline_version=self.pipeline_version)                
                    node['out_topic_dict'][output] = topic_name                    
                    message_buses[output]['in_topic'] = topic_name
                    
            elif actor_type=='detector':
                pass
            
            else:
                output = node['outputs'][0]
                if output == 'end':
                    node['out_topic'] = self.pipeline_out
                else:
                    topic_name = self.topic_tpl.format(from_node=node_name, 
                                                       pipeline_name=self.pipeline_name, 
                                                       pipeline_version=self.pipeline_version)
                    node['out_topic'] = topic_name
                    message_buses[output]['in_topic'] = topic_name
            
            if actor_type!='detector':
                node['error_topic'] = self.error_topic
                node['monitor_topics'] = self.monitor_in
            
                if node['from'][0] == 'begin':
                    node['in_topic'] = self.pipeline_inputs
                
            
            for field in ['class_name', 'actor_type', 'outputs', 'engine_cls', 'from']:
                node.pop(field, None)

        return message_buses
            
