from .actor import MessageBus, Actor, Logger
from atelier_fabrique.message import Message, MessageType, Header
import json, csv

import os, sys, shutil

import uuid
from time import time, sleep
import traceback

from threading import Thread
from atelier_fabrique.constants import API_IN_TOPIC, MONITOR_IN_TOPIC
API_TYPE = 'api_emulator'

class Api(Actor):
    
    def __init__(self, store,
                       in_topics = [API_IN_TOPIC,], 
                       monitor_topics = [MONITOR_IN_TOPIC,],
                       pipeline_name = 'pipeline_name',
                       pipeline_ver='pipeline_ver',
                       input_iterator=None,
                       file_out_dir='./out/data',
                       session_timeout=60.0,
                       is_live=True
                       ):
        
        self.store = store
        
        if not input_iterator:
            raise Exception('input_iterator parameter required')
        
        config = {
            "message_bus": {  
              "pipeline_name": pipeline_name,
              "pipeline_ver": pipeline_ver,
              "in_topic": in_topics,
              "client_id": "api",
              "manage_in_topic": "fabrique.pipeline_manager.in",
              "error_topic": in_topics[0],
              "group_id": 'kafka-router',
              "monitor_topics": monitor_topics,
              "servers": store
              }
          }

        
        super().__init__(config)
        
        self.session_ids = set()
        self.last_time_sent = time()
        self.session_timeout = session_timeout
        
        self.file_out_dir = file_out_dir        
        shutil.rmtree(file_out_dir, ignore_errors=True)
        os.makedirs(file_out_dir)
        
        self.inc_val = 0
        
        self.input_iterator = input_iterator

        self.metric_fields = []
        #self.client_id = 'api'
        #self.in_topics = in_topics
        #self.pipeline_ver = pipeline_ver
        #self.pipeline_name = pipeline_name
        
        #1. лист входных топиков или один топик - неважно
        self.in_topics = in_topics
        #2. вЫходной топик для данных, результатов, скоров и т.д.
        #self.out_topic = f'fabrique.api.{pipeline_name}.{pipeline_ver}.out'
        self.out_topic = 'fabrique.api.{pipeline_name}.{pipeline_version}.out'
        
        #3. топик для ошибок
        #self.error_topic = in_topics[0]
        #4. топик для мониторинга
        #self.monitor_topics = monitor_topics
        
        self.is_live = is_live
 
    
    def send_messages2pipeline(self):
        for data in self.input_iterator:
            session_id = str(uuid.uuid4())
            kafka_key = None
            header = Header(session_id, MessageType.DATA.value, "api",
                            corr_id=kafka_key if kafka_key else None,
                            pipeline_name=self.pipeline_name,
                            pipeline_ver=self.pipeline_ver,
                            fabrique_out_topic=self.out_topic,
                            fabrique_error_topic=self.error_topic,
                            session_start_time=time(),
                            api_type=API_TYPE,
                            is_live=self.is_live)

            self.send_data(header, {'data': data}, topic=self.out_topic)
            
            self.session_ids.add(session_id)
            self.last_time_sent = time()
            
        while self.session_ids:            
            if (time() - self.last_time_sent) > self.session_timeout:
                self.store.stop = True
                print('! Timeout for {} sessions'.format(len(self.session_ids))) 
                return                
            sleep(0.1)
            
        if not self.session_ids:
            print(f'All sessions have been processed (dir {self.file_out_dir})')
        
        sleep_secs = 1
        while(True):
            sleep(sleep_secs) #мониторинги не успевают иногда
            full_len, lengths = self.store.not_empty_topics() 
            if not full_len:
                print('All messages have been processed. Ok!')
                break
            print('{} messages have not been processed yet. Will wait for {} seconds. Nonempty topics: {}.'.format(
                  full_len, sleep_secs, lengths))
            
               
            
        self.close_all()
        
    def close_all(self):
        self.store.stop = True
 
    def batch_callback(self, bodies):
        return bodies
        
    def process_result(self, header, res_body):
        
        i = self.inc_val
        filepath = os.path.join(self.file_out_dir, f'data{i}.bin')
        self.inc_val += 1
        
        with open(filepath,'wb') as f:
            ser_value = res_body['data']
            try:
                f.write(ser_value)
            except TypeError:
                raise TypeError(f"body['data'] of final node must be bytearray not {type(ser_value)}")
        
        self.session_ids.remove(header.session_id)
        
    def run_data(self):
        api_sender = Thread(target=self.send_messages2pipeline)
        api_receiver = Thread(target=self.run)
        
        api_receiver.start()
        api_sender.start()
        api_receiver.join()
        api_sender.join()
        
        


            