from .actor import Actor
import json, os
from datetime import datetime
from atelier_fabrique.constants import API_IN_TOPIC, MONITOR_IN_TOPIC
from atelier_fabrique.constants import MessageFieldsConstants as fld


def_config = {
    "message_bus": {  
      "pipeline_name": "all",
      "pipeline_ver": "all",
      "in_topic": MONITOR_IN_TOPIC,
      "client_id": "monitor",
      "error_topic": None,
      "group_id": None,
      "servers": None,
      }
  }

class Monitor(Actor):
    def __init__(self, store, file_out_name = './out/metrics.ndjson'):   
        config = def_config
        config['message_bus']['servers'] = store
        
        os.makedirs('./out', exist_ok=True)
        
        self.file_out_name = file_out_name
        self.file_out = open(self.file_out_name, 'w')
        
        super().__init__(config)        
    
    """
        Сборщик данных мониторинга
        ничего не отдает, принимает месседжи всех типов, при получении 
    """
    def preprocess_msg(self, msg):
        message = super().preprocess_msg(msg)
        if message:
            #раз колбеку доступен только боди для нужд менеджмента закинем в боди все что может быть интересно
            header = message.header.to_dict()
            kafka_header = message.kafka_header
            body = message.body
            session_id = message.header.session_id
            message.body = dict(body=body, kafka_header=kafka_header, header=header, session_id=session_id)
            return message
 
    def close_all(self):
        self.file_out.close()

    def process_result(self, res_body, session_id):
        return None
    
    def batch_callback(self, batch):
        for message_dict in batch:
            self.callback(message_dict)
    
    def callback(self, message_dict): 
        
        
        session_id = message_dict["session_id"]     
        client_id = message_dict["header"]["source"] #сlient_id актора, приславшего метрики
        source_id = client_id.rsplit('__')[0] #отсечем суффикс клона
        


        pipeline_name = message_dict["header"]["pipeline_name"]
        pipeline_ver = message_dict["header"]["pipeline_ver"] 

        #if not pipeline_name and not pipeline_ver:
        #    return

        body = message_dict['body']
        
        if fld.timestamp in body:
            timestamp = body[fld.timestamp]
        else:
            timestamp = datetime.utcnow().timestamp()
                
        def write_metric(field, pipeline_name, pipeline_ver, source_id, value=None, timestamp=None):            
            self.file_out.write(json.dumps(dict(field=field, value=value, pipeline_name=pipeline_name, 
                                                pipeline_ver=pipeline_ver, source_id=source_id, timestamp=timestamp))+'\n')

                
        if body.get(fld.error):
            write_metric(fld.error, pipeline_name, pipeline_ver, source_id, timestamp=timestamp)
            
        
        if body.get(fld.metrics):

            for field, value in body[fld.metrics].items():
                write_metric(field, pipeline_name, pipeline_ver, source_id, value, timestamp=timestamp)  

        if body.get(fld.alerts):

            #print('! metrics', body['metrics'], pipeline_name, pipeline_ver, source_id)

            for field, value in body[fld.alerts].items():
                #pri.set_custom_summary(value, field, pipeline_name, pipeline_ver, gauge_id)
                write_metric(field, pipeline_name, pipeline_ver, source_id, value, timestamp=timestamp)             

