#embedded inmemory storage for kafka emulation

from collections import deque, namedtuple 
from time import sleep, time

Mes = namedtuple('Mes', ['value', 'topic', 'partition', 'offset', 'timestamp', 'key'])

class Store(object):
    def __init__(self):
        self.stop = False #stop even if topic is nonempty
        self.gentle_stop = False #stop if current topic is empty
        self.topics = {}
        
    def not_empty_topics(self):
        lengths = {}
        full_len = 0
        
        for topic, deq in self.topics.items():
            topic_len = len(deq)
            if topic_len:
                lengths[topic] = topic_len
                full_len += topic_len
        
        return full_len, lengths
        
    def __read_queue(self, topic, limit=1000):
        if self.stop:
            raise StopIteration
        
        msgs = []
        
        if topic not in self.topics:
            return msgs
        
        for k in range(limit):
            
            if self.stop:
                raise StopIteration 
                
            try:
                mes = self.topics[topic].pop()
                msgs.append(mes)
                
            except IndexError:                
                if self.gentle_stop:
                    raise StopIteration
                    
                break
            
        return msgs
        
    def reader_generator(self, topic_lst, empty_sleep=1.0, max_values=1000):
        while True:
            batch = []
            for topic in topic_lst:
                limit = max_values - len(batch)
                if limit <= 0:
                    continue
                
                try:    
                    msgs = self.__read_queue(topic, limit=limit)
                # wanna return None instead of StopIteration from PEP 479
                except StopIteration:  
                    return
                
                batch.extend(msgs)
                
            if not batch:
                sleep(empty_sleep)
                
            yield batch
    
    def produce(self, topic, mes):
        if not topic in self.topics:
            self.topics[topic] = deque()
        self.topics[topic].appendleft(mes)
        
    def produce_kaf_format(self, topic, value, key=b''):
        mes = Mes(value=value,
                  topic=topic, partition=0, offset=0, 
                  timestamp=time(), 
                  key=key)

        self.produce(topic, mes)

        
