try:
    import fabrique.actor as actor
except ImportError:
    import atelier_fabrique.dummy.actor as actor
    
from atelier_fabrique.constants import VALID_ACTOR_TYPES
    
import json
from collections import namedtuple

class BaseActor():
    _outputs = []
    destinations = namedtuple('destinations', [])(*[])
    def __str__(self):
        actor_type = self._actor_type
        class_name = type(self).__name__

        outputs = list(self._outputs)

        return f'{class_name}, type = {actor_type}, outputs = {outputs}' 
    
    @classmethod    
    def to(cls, *destinations): #universal!
        cls._outputs = destinations
        cls.destinations = namedtuple('destinations', destinations)(*destinations)
        return cls
    
    def default_get_batch_result(self, batch):
        res_batch = []
        for body in batch:
            res_batch.append(self.get_result(body))
        return res_batch

    def get_result(self, data):
        raise Exception("This method needs to be defined in a subclass")
    
    def start_actor(self, actor_instance, autorun=True):
        self._actor = actor_instance

        try:
            self._actor.batch_callback = self.get_batch_result  #если определен такой метод, то фигачим батчевый колбек       
        except AttributeError:
            self._actor.batch_callback = self.default_get_batch_result
            
        if autorun:
            self._actor.run()
            
            
class Reducer(BaseActor):
    _actor_type = VALID_ACTOR_TYPES.reducer    
    def start(self, config):
        self.start_actor(actor.Reducer(config))

class Mapper(BaseActor):
    _actor_type = VALID_ACTOR_TYPES.mapper
    def start(self, config):
        self.start_actor(actor.Mapper(config))

class Dispatcher(BaseActor):
    _actor_type = VALID_ACTOR_TYPES.dispatcher
    def start(self, config):
        self.start_actor(actor.Dispatcher(config))
        
class MetricsReducer(BaseActor):
    _actor_type = VALID_ACTOR_TYPES.metrics_reducer    
    def start(self, config):
        self.start_actor(actor.MonitorReducer(config))
        
class Nodes(dict):
    def __init__(self,**kw):
        dict.__init__(self,kw)
        self.__dict__ = self

class Pipeline(object):
    def __init__(self, nodes_lst):
        assert(nodes_lst)
        assert(not set(nodes_lst).intersection({'begin', 'end', 'monitor'})) 
        self.nodes = Nodes()
        for node_id in nodes_lst:
            setattr(self.nodes, node_id, None)        
        
        nodes_lst.extend(['begin', 'end'])  
        self.ids = namedtuple('ids', nodes_lst)(*nodes_lst)
        
