from enum import Enum, unique
from copy import deepcopy
import msgpack, json
import uuid

@unique
class MessageType(Enum):
    ERROR = 'ERROR' #ошибка
    DATA = 'DATA' #data
    MANAGEMENT = 'MANAGEMENT' #указания от pipeline manager
    


class Header(object):
    def __init__(self, session_id, 
                       message_type,
                       source,
                       corr_id=None,
                       pipeline_name=None,
                       pipeline_ver=None,
                       fabrique_out_topic=None,
                       fabrique_error_topic=None,
                       session_start_time=None,
                       api_type=None,
                       is_live=None,
                ):
        
        if not corr_id:
            corr_id = str(uuid.uuid4())
            
        self.corr_id = corr_id
        self.session_id = session_id
        self.pipeline_ver = pipeline_ver
        self.pipeline_name = pipeline_name
        self.message_type = message_type
        self.source = source
        self.fabrique_out_topic = fabrique_out_topic
        self.fabrique_error_topic = fabrique_error_topic
        self.session_start_time = session_start_time
        self.api_type = api_type
        self.is_live = is_live
        
    def to_dict(self):
        return dict(session_id=self.session_id,
                    message_type=self.message_type,
                    source=self.source,
                    corr_id=self.corr_id,
                    pipeline_name=self.pipeline_name,
                    pipeline_ver=self.pipeline_ver,
                    fabrique_out_topic=self.fabrique_out_topic,
                    fabrique_error_topic=self.fabrique_error_topic,
                    session_start_time=self.session_start_time,
                    api_type=self.api_type,
                    is_live=self.is_live
                   )


class Message(object):
    '''класс должен
    1. задавать структуру
    2. парсить месседж кафки
    3. генерить на основе принятого заголовка новый заголовок
    4. генерить типовые заголовки для резалта, ошибки, мониторинга и т.д.
    5. сериализовывать перед отправкой
    + добавил корявенько имя и версию пайплайна'''
    
    def __init__(self, header, body, kafka_header={}):
        self.kafka_header = kafka_header
        self.header = header
        self.body = body     
    
    def serialize(self):       
        header_dict = self.header.to_dict()
        val_dict = dict(header=header_dict, body=self.body)        
        packed = msgpack.dumps(val_dict, use_bin_type=True) #json.dumps(val_dict).encode()
        return packed
    
    @classmethod
    def parse_mes(cls, mes):
        #создавалка месседжа из сообщения кафки
        kafka_header = dict(topic=mes.topic, partition=mes.partition, offset=mes.offset, 
                            timestamp=mes.timestamp, key=mes.key)
        value = mes.value
        #val_dict = json.loads(value.decode())
        val_dict = msgpack.loads(value, encoding='utf-8')
        header_dict = val_dict['header']
        body = val_dict['body']
        body['is_live'] = header_dict.get('is_live')
        ## body['kafka_key'] = key !!!
        header = Header(**header_dict)
        return cls(header, body, kafka_header) 
    
    @classmethod
    def gen_message_by_header(cls, header, body, message_type=None, source=None, preserve_corr_id=True):
        if not preserve_corr_id:
            header.corr_id = str(uuid.uuid4())
        if message_type:
            header.message_type = message_type
        if source:
            header.source = source
        return cls(header, body)