from atelier_fabrique.validators.pipeline_validator import validate_pipeline
from atelier_fabrique.config_generator import ActorConfigs

from atelier_fabrique.dummy.api import Api
from atelier_fabrique.dummy.monitor import Monitor
from atelier_fabrique.dummy.store import Store
from threading import Thread

from atelier_fabrique.dummy.logger import Logger

import importlib.machinery
import os, sys, json



def start_node(instance, config):
    instance.start(config)

class Project():
    def __init__(self, proj_path):
        #path = os.path.join(proj_path, 'pipeline.py')
        self.proj_path = os.path.abspath(proj_path)
        #sys.path.append(self.proj_path)
        
        self.starter_pth = os.getcwd()
        
        os.chdir(self.proj_path)
        #print(self.proj_path)
        
        loader = importlib.machinery.SourceFileLoader('pipeline', os.path.join(self.proj_path, 'pipeline.py'))
        mod = loader.load_module()
        pipeline = mod.pipeline
        
        self.nodes = validate_pipeline(pipeline)
        
        self.save_nodes_for_prod()
        
        self.pipeline = pipeline 
        self.store = Store()
        #print(self.store)
    
    def save_nodes_for_prod(self):
        
        with open(os.path.join(self.proj_path, 'pipeline.json'), 'w') as fp:
            json.dump({'nodes': self.nodes, 'python_ver':sys.version_info}, fp, indent=2)        
         
        print(f'pipeline.json is generated in {self.proj_path}')

    
    def start_pipeline(self, input_iterator=None, session_timeout=100.0):    
        nodes = self.nodes
        store = self.store
        pipeline = self.pipeline
        
        #logger = Logger.make_logger('all')
        

        cfg = ActorConfigs(nodes, servers=store)

        cfg.make_message_bus_configs()

        
        threads = []

        for nodename, val in nodes.items():
            
            instance = pipeline.nodes[nodename]()

            config = cfg.nodename2config(nodename)
            #print(config)
            thread = Thread(target=start_node, args=(instance, config))
            threads.append(thread)

        os.chdir(self.starter_pth) #.ndjsons in dev dir  
          
        monitor = Monitor(store)    
            
        api = Api(self.store, session_timeout=session_timeout, input_iterator=input_iterator)

        api.out_topic = cfg.pipeline_inputs[0]    
        #print(api.out_topic)    
            
        mon_thread = Thread(target=monitor.run)
        threads.append(mon_thread)

        for thread in threads:
            thread.start()

        api.run_data()
        #print("api.run_data()")

        for thread in threads:
            thread.join()