from atelier_fabrique.validators.pipeline_validator import validate_pipeline
from atelier_fabrique.config_generator import ActorConfigs

from atelier_fabrique.dummy.api import Api
from atelier_fabrique.dummy.monitor import Monitor
from atelier_fabrique.dummy.store import Store
from threading import Thread

import importlib.machinery
import os

store = Store()

def start_node(instance, config):
    instance.start(config)
    
def start_pipeline(proj_path, input_generator=None, session_timeout=10.0):
    path = os.path.join(proj_path, 'pipeline.py')
    
    loader = importlib.machinery.SourceFileLoader('pipeline', path)
    mod = loader.load_module()
    pipeline = mod.pipeline

    nodes, start_node_name = validate_pipeline(pipeline)

    cfg = ActorConfigs(nodes, servers=store)

    cfg.make_message_bus_configs()

    api = Api(store, session_timeout=session_timeout, input_generator=input_generator)
    api.out_topic = cfg.pipeline_inputs[0]
    
    monitor = Monitor(store)
    
    threads = []

    for nodename, val in nodes.items():
        instance = val['engine_instance']
        config = cfg.nodename2config(nodename)
        thread = Thread(target=start_node, args=(instance, config))
        threads.append(thread)
     
    mon_thread = Thread(target=monitor.run)
    threads.append(mon_thread)
    
    for thread in threads:
        thread.start()
    
    api.run_data() 

    for thread in threads:
        thread.join()

